@@ -41,8 +41,9 @@
 #include "config.h"
 #endif
 
-#include "clutter-fixed.h"
 #include "clutter-backend.h"
+#include "clutter-debug.h"
+#include "clutter-fixed.h"
 #include "clutter-private.h"
 
 G_DEFINE_ABSTRACT_TYPE (ClutterBackend, clutter_backend, G_TYPE_OBJECT);
@@ -57,6 +58,8 @@ struct _ClutterBackendPrivate
   guint double_click_distance;
 
   ClutterFixed resolution;
+
+  cairo_font_options_t *font_options;
 };
 
 static void
@@ -73,6 +76,8 @@ clutter_backend_dispose (GObject *gobject)
       clutter_context->events_queue = NULL;
     }
 
+  clutter_backend_set_font_options (CLUTTER_BACKEND (gobject), NULL);
+
   G_OBJECT_CLASS (clutter_backend_parent_class)->dispose (gobject);
 }
 
@@ -95,14 +100,6 @@ clutter_backend_init (ClutterBackend *backend)
   priv->resolution = -1.0;
 }
 
-ClutterActor *
-_clutter_backend_get_stage (ClutterBackend *backend)
-{
-  g_return_val_if_fail (CLUTTER_IS_BACKEND (backend), NULL);
-
-  return CLUTTER_BACKEND_GET_CLASS (backend)->get_stage (backend);
-}
-
 void
 _clutter_backend_add_options (ClutterBackend *backend,
                               GOptionGroup   *group)
@@ -146,31 +143,89 @@ _clutter_backend_post_parse (ClutterBackend  *backend,
   return TRUE;
 }
 
-gboolean
-_clutter_backend_init_stage (ClutterBackend  *backend,
-                             GError         **error)
+ClutterActor *
+_clutter_backend_create_stage (ClutterBackend  *backend,
+                               ClutterStage    *wrapper,
+                               GError         **error)
 {
+  ClutterMainContext  *context;
   ClutterBackendClass *klass;
+  ClutterActor        *stage = NULL;
 
   g_return_val_if_fail (CLUTTER_IS_BACKEND (backend), FALSE);
+  g_return_val_if_fail (CLUTTER_IS_STAGE (wrapper), FALSE);
+
+  context = clutter_context_get_default ();
+
+  if (!context->stage_manager)
+    context->stage_manager = clutter_stage_manager_get_default ();
 
   klass = CLUTTER_BACKEND_GET_CLASS (backend);
-  if (klass->init_stage)
-    return klass->init_stage (backend, error);
+  if (klass->create_stage)
+    stage = klass->create_stage (backend, wrapper, error);
 
-  return TRUE;
+  if (!stage)
+    return NULL;
+
+  g_assert (CLUTTER_IS_STAGE_WINDOW (stage));
+  _clutter_stage_set_window (wrapper, CLUTTER_STAGE_WINDOW (stage));
+  _clutter_stage_manager_add_stage (context->stage_manager, wrapper);
+
+  return stage;
 }
 
 void
-_clutter_backend_redraw (ClutterBackend *backend)
+_clutter_backend_redraw (ClutterBackend *backend,
+                         ClutterStage   *stage)
 {
   ClutterBackendClass *klass;
 
   klass = CLUTTER_BACKEND_GET_CLASS (backend);
-  if (G_LIKELY(klass->redraw))
-    klass->redraw (backend);
+  if (G_LIKELY (klass->redraw))
+    klass->redraw (backend, stage);
 }
 
+void
+_clutter_backend_ensure_context (ClutterBackend *backend,
+                                 ClutterStage   *stage)
+{
+  ClutterBackendClass *klass;
+  static ClutterStage *current_context_stage = NULL;
+
+  g_return_if_fail (CLUTTER_IS_BACKEND (backend));
+  g_return_if_fail (CLUTTER_IS_STAGE (stage));
+
+  if (current_context_stage != stage || !CLUTTER_ACTOR_IS_REALIZED (stage))
+    {
+      if (!CLUTTER_ACTOR_IS_REALIZED (stage))
+        {
+          CLUTTER_NOTE (MULTISTAGE, "Stage is not realized, unsetting");
+          stage = NULL;
+        }
+      else
+        CLUTTER_NOTE (MULTISTAGE, "Setting the new stage [%p]", stage);
+      
+      klass = CLUTTER_BACKEND_GET_CLASS (backend);
+      if (G_LIKELY (klass->ensure_context))
+        klass->ensure_context (backend, stage);
+      
+      /* FIXME: With a NULL stage and thus no active context it may make more
+       * sense to clean the context but then re call with the default stage 
+       * so at least there is some kind of context in place (as to avoid
+       * potential issue of GL calls with no context)
+       */
+      current_context_stage = stage;
+
+#if 0
+      if (stage)
+        CLUTTER_SET_PRIVATE_FLAGS (stage, CLUTTER_ACTOR_SYNC_MATRICES);
+#endif
+    }
+  else
+    CLUTTER_NOTE (MULTISTAGE, "Stage is the same");
+}
+
+
 ClutterFeatureFlags
 _clutter_backend_get_features (ClutterBackend *backend)
 {
@@ -333,6 +388,10 @@ clutter_backend_set_resolution (ClutterBackend *backend,
   fixed_dpi = CLUTTER_FLOAT_TO_FIXED (dpi);
   if (priv->resolution != fixed_dpi)
     priv->resolution = fixed_dpi;
+
+  if (CLUTTER_CONTEXT ()->font_map)
+    pango_clutter_font_map_set_resolution (CLUTTER_CONTEXT ()->font_map,
+					   CLUTTER_FIXED_TO_FLOAT (fixed_dpi));
 }
 
 /**
@@ -354,3 +413,74 @@ clutter_backend_get_resolution (ClutterBackend *backend)
 
   return CLUTTER_FIXED_TO_FLOAT (backend->priv->resolution);
 }
+
+/**
+ * clutter_backend_set_font_options:
+ * @backend: a #ClutterBackend
+ * @options: Cairo font options for the backend, or %NULL
+ *
+ * Sets the new font options for @backend. If @options is %NULL,
+ * the first following call to clutter_backend_get_font_options()
+ * will return the default font options for @backend.
+ *
+ * This function is intended for actors creating a Pango layout
+ * using the PangoCairo API.
+ *
+ * Since: 0.8
+ */
+void
+clutter_backend_set_font_options (ClutterBackend       *backend,
+                                  cairo_font_options_t *options)
+{
+  ClutterBackendPrivate *priv;
+
+  g_return_if_fail (CLUTTER_IS_BACKEND (backend));
+
+  priv = backend->priv;
+
+  if (priv->font_options != options)
+    {
+      if (priv->font_options)
+        cairo_font_options_destroy (priv->font_options);
+
+      if (options)
+        priv->font_options = cairo_font_options_copy (options);
+      else
+        priv->font_options = NULL;
+    }
+}
+
+/**
+ * clutter_backend_get_font_options:
+ * @backend: a #ClutterBackend
+ *
+ * Retrieves the font options for @backend.
+ *
+ * Return value: the font options of the #ClutterBackend
+ *
+ * Since: 0.8
+ */
+cairo_font_options_t *
+clutter_backend_get_font_options (ClutterBackend *backend)
+{
+  ClutterBackendPrivate *priv;
+
+  g_return_val_if_fail (CLUTTER_IS_BACKEND (backend), NULL);
+
+  priv = backend->priv;
+
+  if (G_LIKELY (priv->font_options))
+    return priv->font_options;
+
+  priv->font_options = cairo_font_options_create ();
+
+  cairo_font_options_set_hint_style (priv->font_options,
+                                     CAIRO_HINT_STYLE_NONE);
+  cairo_font_options_set_subpixel_order (priv->font_options,
+                                         CAIRO_SUBPIXEL_ORDER_DEFAULT);
+  cairo_font_options_set_antialias (priv->font_options,
+                                    CAIRO_ANTIALIAS_DEFAULT);
+
+  return priv->font_options;
+}
+
