@@ -77,11 +77,8 @@ struct _ClutterTexturePrivate
 {
   gint                         width;
   gint                         height;
-  guint                        sync_actor_size : 1;
   gint                         max_tile_waste;
   ClutterTextureQuality        filter_quality;
-  guint                        repeat_x : 1;
-  guint                        repeat_y : 1;
   CoglHandle                   texture;
   gboolean                     no_slice;
 
@@ -94,6 +91,9 @@ struct _ClutterTexturePrivate
   guint                        local_data_has_alpha;
   guchar                      *local_data;
 
+  guint                        sync_actor_size : 1;
+  guint                        repeat_x : 1;
+  guint                        repeat_y : 1;
   guint                        in_dispose : 1;
   guint                        keep_aspect_ratio : 1;
 };
@@ -166,25 +166,6 @@ clutter_texture_quality_to_cogl_mag_filter (ClutterTextureQuality buf_filter)
   return 0;
 }
 
-static ClutterTextureQuality
-cogl_filters_to_clutter_texture_quality (COGLenum min,
-                                         COGLenum mag)
-{
-  switch (min)
-    {
-      case CGL_NEAREST:
-         g_assert (mag == min); /* just for sanity */
-         return CLUTTER_TEXTURE_QUALITY_LOW;
-      case CGL_LINEAR:
-         g_assert (mag == min); /* just for sanity */
-         return CLUTTER_TEXTURE_QUALITY_MEDIUM;
-      case CGL_LINEAR_MIPMAP_LINEAR:
-         g_assert (mag == CGL_LINEAR); /* just for sanity */
-         return CLUTTER_TEXTURE_QUALITY_HIGH;
-    }
-  return 0;
-}
-
 static void
 texture_free_gl_resources (ClutterTexture *texture)
 {
@@ -571,6 +552,13 @@ clutter_texture_paint (ClutterActor *self)
         clutter_shader_set_is_enabled (shader, TRUE);
     }
 
+  /* A clone may need to fire above if were a TFP/FBO but not visible.
+   * Ultimatly needs some reworking with maybe an extra prepare_paint
+   * method or some such.
+  */
+  if (CLUTTER_PRIVATE_FLAGS(self) & CLUTTER_TEXTURE_IN_CLONE_PAINT)
+    return;
+
   CLUTTER_NOTE (PAINT,
                 "painting texture '%s'",
 		clutter_actor_get_name (self) ? clutter_actor_get_name (self)
@@ -1386,10 +1374,12 @@ clutter_texture_set_filter_quality (ClutterTexture        *texture,
   priv = texture->priv;
 
   old_quality = clutter_texture_get_filter_quality (texture);
+
   if (filter_quality != old_quality)
     {
       priv->filter_quality = filter_quality;
 
+      /* Is this actually needed - causes problems with TFP mipmaps */
       if (priv->texture != COGL_INVALID_HANDLE)
 	cogl_texture_set_filters (priv->texture,
              clutter_texture_quality_to_cogl_min_filter (priv->filter_quality),
@@ -1399,10 +1389,17 @@ clutter_texture_set_filter_quality (ClutterTexture        *texture,
            filter_quality == CLUTTER_TEXTURE_QUALITY_HIGH) &&
            CLUTTER_ACTOR_IS_REALIZED (texture))
         {
-          clutter_texture_unrealize (CLUTTER_ACTOR (texture));
-          clutter_texture_realize (CLUTTER_ACTOR (texture));
+          gboolean was_visible;
+
+          was_visible = CLUTTER_ACTOR_IS_VISIBLE (CLUTTER_ACTOR (texture));
+
+          clutter_actor_unrealize (CLUTTER_ACTOR (texture));
+          clutter_actor_realize (CLUTTER_ACTOR (texture));
+
+          if (was_visible)
+            clutter_actor_show (CLUTTER_ACTOR (texture));
         }
-      
+
       g_object_notify (G_OBJECT (texture), "filter-quality");
 
       if (CLUTTER_ACTOR_IS_VISIBLE (texture))
@@ -1429,15 +1426,7 @@ clutter_texture_get_filter_quality (ClutterTexture *texture)
 
   priv = texture->priv;
 
-  if (priv->texture == COGL_INVALID_HANDLE)
-    return texture->priv->max_tile_waste;
-  else
-    /* If we have a valid texture handle then use the filter quality
-       from that instead */
-
-  return cogl_filters_to_clutter_texture_quality (
-      cogl_texture_get_min_filter (texture->priv->texture),
-      cogl_texture_get_mag_filter (texture->priv->texture));
+  return priv->filter_quality;
 }
 
 /**
