@@ -314,7 +314,13 @@ cogl_enable (gulong flags)
   cogl_toggle_flag (ctx, flags,
 		    COGL_ENABLE_TEXTURE_2D,
 		    GL_TEXTURE_2D);
-  
+
+#ifdef GL_TEXTURE_RECTANGLE_ARB
+  cogl_toggle_flag (ctx, flags,
+		    COGL_ENABLE_TEXTURE_RECT,
+		    GL_TEXTURE_RECTANGLE_ARB);
+#endif
+
   cogl_toggle_client_flag (ctx, flags,
 			   COGL_ENABLE_VERTEX_ARRAY,
 			   GL_VERTEX_ARRAY);
@@ -1101,6 +1107,13 @@ _cogl_features_init ()
   if (max_clip_planes >= 4)
     flags |= COGL_FEATURE_FOUR_CLIP_PLANES;
 
+  /* This should always be available because it is defined in GL 1.2,
+     but we can't call it directly because under Windows functions >
+     1.1 aren't exported */
+  ctx->pf_glDrawRangeElements =
+        (COGL_PFNGLDRAWRANGEELEMENTSPROC)
+        cogl_get_proc_address ("glDrawRangeElements");
+
   /* Cache features */
   ctx->feature_flags = flags;
   ctx->features_cached = TRUE;
