@@ -450,6 +450,9 @@ cogl_gles2_wrapper_get_uniforms (GLuint program,
 
   uniforms->alpha_test_ref_uniform
     = glGetUniformLocation (program, "alpha_test_ref");
+
+  uniforms->texture_unit_uniform
+    = glGetUniformLocation (program, "texture_unit");
 }
 
 void
@@ -801,8 +804,8 @@ cogl_wrap_glColorPointer (GLint size, GLenum type, GLsizei stride,
 			 GL_FALSE, stride, pointer);
 }
 
-void
-cogl_wrap_glDrawArrays (GLenum mode, GLint first, GLsizei count)
+static void
+cogl_wrap_prepare_for_draw (void)
 {
   CoglGles2WrapperProgram *program;
 
@@ -875,6 +878,10 @@ cogl_wrap_glDrawArrays (GLenum mode, GLint first, GLsizei count)
 	glUniform1f (program->uniforms.alpha_test_ref_uniform,
 		     w->alpha_test_ref);
 
+      if ((w->dirty_uniforms & COGL_GLES2_DIRTY_TEXTURE_UNIT)
+          && program->uniforms.texture_unit_uniform != -1)
+        glUniform1i (program->uniforms.texture_unit_uniform, 0);
+
       w->dirty_uniforms = 0;
     }
 
@@ -904,10 +911,25 @@ cogl_wrap_glDrawArrays (GLenum mode, GLint first, GLsizei count)
       
       w->dirty_custom_uniforms = 0;
     }
+}
+
+void
+cogl_wrap_glDrawArrays (GLenum mode, GLint first, GLsizei count)
+{
+  cogl_wrap_prepare_for_draw ();
 
   glDrawArrays (mode, first, count);
 }
 
+void
+cogl_wrap_glDrawElements (GLenum mode, GLsizei count, GLenum type,
+                          const GLvoid *indices)
+{
+  cogl_wrap_prepare_for_draw ();
+
+  glDrawElements (mode, count, type, indices);
+}
+
 void
 cogl_gles2_wrapper_bind_texture (GLenum target, GLuint texture,
 				 GLenum internal_format)
