@@ -351,7 +351,11 @@ message_translate (ClutterBackend *backend,
   switch (msg->message)
     {
     case WM_SIZE:
-      if (!stage_win32->is_foreign_win)
+      if (!stage_win32->is_foreign_win
+	  /* Ignore size changes resulting from the stage being
+	     minimized - otherwise the window size will be set to
+	     0,0 */
+	  && msg->wParam != SIZE_MINIMIZED)
 	{
 	  WORD new_width = LOWORD (msg->lParam);
 	  WORD new_height = HIWORD (msg->lParam);
@@ -467,11 +471,17 @@ message_translate (ClutterBackend *backend,
 
       event->type = CLUTTER_SCROLL;
       event->scroll.time = msg->time;
-      event->scroll.x = GET_X_LPARAM (msg->lParam);
-      event->scroll.y = GET_Y_LPARAM (msg->lParam);
       event->scroll.modifier_state
 	= get_modifier_state (LOWORD (msg->wParam));
 
+      /* conversion to window coordinates is required */
+      {
+	POINT pt = { GET_X_LPARAM (msg->lParam), GET_Y_LPARAM (msg->lParam) };
+	ScreenToClient (msg->hwnd, &pt);
+	event->scroll.x = pt.x;
+	event->scroll.y = pt.y;
+      }
+
       if (stage_win32->scroll_pos >= WHEEL_DELTA)
 	{
 	  event->scroll.direction = CLUTTER_SCROLL_UP;
